/**
 * @file
 * jQuery based sorting function.
 */
Drupal.behaviors.nodeGalleryInitSorting = function(context) {
  // hide the weight dropdowns
  $('*[id^=edit-images-sort][id$=-wrapper]').each(function() {
    $(this).hide();
  });
  // enable drag and drop
  $("div.node-gallery-sort-images-grid-sortable").sortable({
    update: nodeGalleryUpdatePositions
  });
  $("div.node-gallery-sort-images-grid-sortable").disableSelection();
  // enable presets
  $('div.node-gallery-sort-presets a:not(.ng3-processed)', context).addClass('ng3-processed').bind('click', nodeGalleryPresetClickHandler);
};

var nodeGalleryUpdatePositions = function() {
  var self = this;
  this.newOrder = [];
  $("div.node-gallery-sort-images-grid-sortable").children().each(function(index, element) {
    self.newOrder.push(Drupal.settings.node_gallery.original_sort[element.id.substring(element.id.lastIndexOf('-')+1)]);
    $('#edit-'+element.id+' option[value='+index+']').attr("selected", true);
  });
  changed = (self.newOrder.join('') != Drupal.settings.node_gallery.original_sort.join(""));
  // show warning on first change and remove it if original was restored
  if ($("#node-gallery-sort-images-grid").hasClass("markedChanged")) {
    if (!changed) {
      // user restored original order, remove warning
      $("#node-gallery-sort-images-grid").removeClass("markedChanged");
      $("div.node-gallery-order-changed-warning").fadeOut('slow', function() { $(this).remove(); });
    }
  } else if (changed) {
    $(Drupal.theme('nodeGalleryOrderChangedWarning')).insertAfter($("#node-gallery-sort-images-grid")).hide().fadeIn('slow');
    $("#node-gallery-sort-images-grid").addClass("markedChanged");
  }
}

var nodeGalleryPresetClickHandler = function() {
  op = this.id.substring(this.id.lastIndexOf('-')+1);
  url = Drupal.settings.basePath+'node-gallery/json/gallery/'+Drupal.settings.node_gallery.gid+'/sort/'+op;
  if ($(this).hasClass("node-gallery-sort-active")) {
    url += '/1';
  }
  $(this).toggleClass("node-gallery-sort-active").siblings().removeClass("node-gallery-sort-active");
  $.get(url, null, nodeGallerySortByNidArray);
  return false;
}

var nodeGallerySortByNidArray = function(response) {
  var result, index;
  result = Drupal.parseJson(response);
  for (key in result) {
    index = jQuery.inArray(result[key], Drupal.settings.node_gallery.original_sort);
    $("div.node-gallery-sort-images-grid-sortable").append($('#images-sort-'+index));
  }
  nodeGalleryUpdatePositions();
  $("div.node-gallery-sort-images-grid-sortable").sortable("refreshPositions");
}

Drupal.theme.prototype.nodeGalleryOrderChangedWarning = function () {
  return '<div class="warning node-gallery-order-changed-warning"><span class="warning tabledrag-changed">*</span> ' + Drupal.t("Changes made in this table will not be saved until the form is submitted.") + '</div>';
};
