<?php
// $Id: theme.inc,v 1.2.2.39 2010/11/27 17:08:52 justintime Exp $

/**
 * @file
 * Node gallery theme functions
 */

function theme_node_gallery_manage_images_form($form) {
  // get fieldname to retrieve the filepath for the thumbnail without loading the node
  $relationship = node_gallery_get_relationship($form['#gallery']->type);
  $field_name = $relationship['imagefield_name'];
  $thumb_imagecache = $form['#thumb_imagecache'];
  $header = array(t('Delete'), t('Preview'), t('Edit'), t('Cover'));
  foreach (element_children($form['images']) as $nid) {
    $element = &$form['images'][$nid];
    $filepath = node_gallery_get_image_filepath($element['edit_form']['nid']['#value'], $field_name);

    $row = array();
    $row[] = drupal_render($element['remove']);
    $row[] = '<span>'. l(theme('imagecache', 'node-gallery-admin-thumbnail', $filepath), 'node/'. $nid .'/edit', array('html' => TRUE)) .'</span>';
    $edit = drupal_render($element['edit_form']);
    if (!empty($element['gid'])) {
      $edit .= drupal_render($element['gid']);
    }
    $row[] = $edit;
    if ($form['is_cover']) {
      $row[] = drupal_render($form['is_cover'][$nid]);
    }
    $rows[] = array('data' => $row);
  }
  $output = theme('table', $header, $rows, array('id' => 'upload-attachments'));
  $output .= drupal_render($form);
  return $output;
}

function template_preprocess_node_gallery_image_navigator(&$variables) {
  $extra = array();
  $relationship = node_gallery_get_relationship(NULL, $variables['image']->type);
  if ($relationship['settings']['node_images_page_fragment']) {
    $extra = array('fragment' => 'node-inner');
  }
  if (isset($variables['navigator']['prev_nid'])) {
    $variables['prev_link'] = url('node/'. $variables['navigator']['prev_nid'], $extra);
    $variables['first_link'] = url('node/'. $variables['navigator']['first_nid'], $extra);
  }
  if (isset($variables['navigator']['next_nid'])) {
    $variables['next_link'] = url('node/'. $variables['navigator']['next_nid'], $extra);
    $variables['last_link'] = url('node/'. $variables['navigator']['last_nid'], $extra);
  }
  $variables['gallery_link'] = url('node/'. $variables['navigator']['gallery_nid']);
  $gallery = node_load($variables['navigator']['gallery_nid']);
  $variables['gallery_title'] = check_plain($gallery->title);
}

/**
 * Template preprocess function for the sort images grid.
 *
 * @param <type> $variables
 *
 */
function template_preprocess_node_gallery_sort_images_grid(&$variables) {
  $original_sort = array();
  foreach ($variables['images'] as $image) {
    $original_sort[] = $image['nid'];
    $variables['items'] .= theme('node_gallery_sort_images_grid_item', $image);
  }
  drupal_add_js(array('node_gallery' => array('original_sort' => $original_sort)), 'setting');
}

/**
 * Imports the image properties into the variables namespace for easier theming.
 *
 * @param <type> $variables
 */
function template_preprocess_node_gallery_sort_images_grid_item(&$variables) {
  $variables += $variables['image'];
}

function theme_node_gallery_sort_images_form($form) {
  $output = '';
  if ($form['jquery_ui_integration']['#value']) {
    jquery_ui_add(array('ui.draggable', 'ui.droppable', 'ui.sortable'));
    drupal_add_js(drupal_get_path('module', 'node_gallery') .'/js/ng_sort_jqueryui.js');
    drupal_add_js(array('node_gallery' => array('gid' => $form['gid']['#value'])), 'setting');
    $output .= theme('node_gallery_sort_images_grid', $form['#images']);
  }
  else {
    $rows = array();
    $header = array('&uarr;&darr;', t('Preview'), t('Title'), t('Post date'), t('Updated date'), t('Published'), t('Weight'));
    $i = 0;
    foreach ($form['#images'] as $image) {
      $published = ($image['status']) ? t('Yes') : t('No');
      $row = array(
        // The placeholder for the tabledrag icon
        '',
        $image['admin_thumb'],
        $image['title'],
        format_date($image['created'], 'small'),
        format_date($image['changed'], 'small'),
        $published,
        // the weight drop down
        drupal_render($form['images-sort-'. $i]),
      );
      $rows[] = array('data' => $row, 'class' => 'draggable');
      $i++;
    }
    // this will replace the weight drop downs with drag&drop when javascript is available
    drupal_add_tabledrag('sort-images-table', 'order', 'sibling', 'sort');
    $output .= theme('table', $header, $rows, array('id' => 'sort-images-table'));
  }
  // render the left over elements, if any
  $output .= drupal_render($form);
  return $output;
}

/**
 * Node gallery formatter hooks and callbacks.
 */
function theme_node_gallery_formatter_nextimagelink($element) {
  // Inside a view $element may contain null data. In that case, just return.
  if (empty($element['#item']['fid'])) {
    return '';
  }

  // Extract the preset name from the formatter name.
  // Note: this is the default imagecache way, no
  $presetname = substr($element['#formatter'], 0, strrpos($element['#formatter'], '_'));

  // If node type is not in a gallery relationship or gid is not set, just render the default formatter.
  $node = $element['#node'];
  if (!in_array($node->type, (array)node_gallery_get_types('image'))
          || !isset($node->gid)) {
    return theme_imagecache_formatter_default($element);
  }

  $next = node_gallery_get_next_image($node->gid, $node->nid);
  $imagetag = theme_imagecache_formatter_default($element);
  if ($next !== NULL) {
    $class = 'imagefield image-nextimagelink imagefield-'. $element['#field_name'];
    return l($imagetag, 'node/'. $next, array('attributes' => array('class' => $class), 'html' => TRUE));
  }
  else {
    // no link on the last image
    return $imagetag;
  }
}
